@@ -51,7 +51,7 @@ class Controller(object):
 
     @_report_exception
     def load_sound(self, filename):
-        self._player.pause()
+        self._player.stop()
         self._sound = edit.Sound(filename)
         self._graph.set_sound(self._sound)
         self._player.set_sound(self._sound)
@@ -81,7 +81,7 @@ class Controller(object):
         sound = self._sound
         if not sound.is_saved() and not sound.is_fresh() and not force:
             raise FileNotSaved
-        self._player.pause()
+        self._player.stop()
 
     def play(self):
         start, end = self._selection.get()
@@ -91,11 +91,18 @@ class Controller(object):
         self._player.end = end
         self._player.thread_play()
 
-    def pause(self):
-        self._player.pause()
+    def stop(self):
+        self._player.stop()
+
+    def toggle_play(self):
+        if self._player._playing:
+            self.stop()
+        else:
+            self.play()
 
     def goto_start(self):
         self._selection.set(0, 0)
+        self._graph.move_to(0)
 
     def goto_end(self):
         end = len(self._sound.frames)
@@ -136,9 +143,20 @@ class Controller(object):
 
     def undo(self):
         self._sound.undo()
+        self.fix_selection()
 
     def redo(self):
         self._sound.redo()
+        self.fix_selection()
+
+    def fix_selection(self):
+        start, end = self._selection.get()
+        n = len(self._sound.frames)
+        if end > n:
+            # Selection has become invalid.
+            start = min(start, n)
+            end = n
+            self._selection.set(start, end)
 
     def select_all(self):
         self._selection.select_all()
@@ -156,14 +174,6 @@ class Controller(object):
         else:
             self._graph.zoom_out_full()
 
-    def list_effects(self):
-        l = effect.effects.keys()
-        l.sort()
-        return l
-
-    def list_extensions(self):
-        return edit.list_extensions()
-
     def effect(self, name):
         if self._selection.selected():
             start, end = self._selection.get()
@@ -186,5 +196,47 @@ def test_Controller():
     ctrl.open('../sounds/test1.wav')
     assert ctrl._sound != None
 
+def test_fix_selection():
+    from mock import Fake, Mock
+    from selection import Selection
+    import numpy
+
+    # Undo
+    graph = Mock({})
+    graph.changed = Fake()
+    selection = Selection(graph, Fake())
+    sound = edit.Sound()
+    sound.frames = numpy.array(range(1000))
+    ctrl = Controller(sound, Fake(), Fake(), selection)
+    frames = sound.frames
+    selection.set(0, 999)
+    ctrl.copy()
+    ctrl.paste()
+    selection.set(1500, 1500)
+    ctrl.undo()
+    ctrl.paste()
+    ctrl.undo()
+    assert sound.frames.tolist() == frames.tolist()
+
+    # Redo
+    graph = Mock({})
+    graph.changed = Fake()
+    selection = Selection(graph, Fake())
+    sound = edit.Sound()
+    sound.frames = numpy.array(range(1000))
+    ctrl = Controller(sound, Fake(), Fake(), selection)
+    frames = sound.frames
+    selection.set(10, 999)
+    ctrl.cut()
+    ctrl.undo()
+    selection.set(900, 900)
+    ctrl.redo()
+    frames = sound.frames 
+    ctrl.paste()
+    ctrl.undo()
+    assert sound.frames.tolist() == frames.tolist()
+    
+
 if __name__ == "__main__":
     test_Controller()
+    test_fix_selection()
